-- FS25_MonumentRestoration/scripts/PlaceableBankAccount.lua

PlaceableBankAccount = {}
PlaceableBankAccount.modName = g_currentModName or "FS25_NAS1000BMP"
PlaceableBankAccount.modDirectory = g_currentModDirectory

function PlaceableBankAccount.prerequisitesPresent(specializations)
    return true
end

function PlaceableBankAccount.initSpecialization()
    local schema = Placeable.xmlSchema
    schema:setXMLSpecializationType("BankAccount")
    
    local baseXmlPath = "placeable.bankAccount"
    
    schema:register(XMLValueType.STRING, baseXmlPath .. "#triggerNode", "Узел триггера")
    schema:register(XMLValueType.FLOAT, baseXmlPath .. "#depositAmount", "Сумма депозита за одно пополнение", 5000)
    
    local savegameSchema = Placeable.xmlSchemaSavegame
    savegameSchema:register(XMLValueType.FLOAT, "placeables.placeable(?)." .. PlaceableBankAccount.modName .. ".bankAccount#balance", "Текущий баланс счёта", 0)
    savegameSchema:register(XMLValueType.INT, "placeables.placeable(?)." .. PlaceableBankAccount.modName .. ".bankAccount#lastInterestDay", "Последний день начисления процентов", 0)
    
    schema:setXMLSpecializationType()
    print("[PlaceableBankAccount] Зарегистрированы XML-пути")
    
    -- Регистрация консольной команды
    addConsoleCommand("reloadBankAccountHUD", "Reloads the BankAccountHUD GUI for testing", "reloadBankAccountHUD", PlaceableBankAccount)
end

function PlaceableBankAccount.reloadBankAccountHUD()
    if not g_gui then
        print("[PlaceableBankAccount] Ошибка: g_gui не инициализирован")
        return
    end
    
    local guiPath = PlaceableBankAccount.modDirectory .. "gui/BankAccountHUD.xml"
    print("[PlaceableBankAccount] Перезагрузка GUI из: " .. tostring(guiPath))
    
    if not fileExists(guiPath) then
        print("[PlaceableBankAccount] Ошибка: Файл GUI не найден по пути: " .. tostring(guiPath))
        return
    end
    
    -- Сохраняем текущий объект банка, если HUD открыт
    local currentBank = nil
    local wasOpen = false
    if BankAccountHUD.INSTANCE and BankAccountHUD.INSTANCE.bank then
        currentBank = BankAccountHUD.INSTANCE.bank
        wasOpen = true
        BankAccountHUD.INSTANCE:close()
        print("[PlaceableBankAccount] Текущий HUD закрыт перед перезагрузкой")
    end
    
    -- Удаляем старый экземпляр HUD
    if BankAccountHUD.INSTANCE then
        BankAccountHUD.INSTANCE:delete()
        BankAccountHUD.INSTANCE = nil
        print("[PlaceableBankAccount] Старый экземпляр HUD удалён")
    end
    
    -- Удаляем старую регистрацию GUI
    if g_gui.guis["BankAccountHUD"] then
        g_gui.guis["BankAccountHUD"] = nil
        print("[PlaceableBankAccount] Старая регистрация GUI удалена")
    end
    
    -- Перезагружаем GUI с помощью метода register из BankAccountHUD
    local success = BankAccountHUD.register()
    if not success then
        print("[PlaceableBankAccount] Ошибка: Не удалось перезагрузить HUD")
        return
    end
    
    -- Если HUD был открыт, открываем его заново
    if wasOpen and currentBank then
        BankAccountHUD.INSTANCE.bank = currentBank
        g_gui:showDialog("BankAccountHUD")
        print("[PlaceableBankAccount] HUD перезагружен и открыт заново для банка: " .. tostring(currentBank))
    else
        print("[PlaceableBankAccount] HUD перезагружен, но не был открыт")
    end
end

function PlaceableBankAccount.registerFunctions(placeableType)
    SpecializationUtil.registerFunction(placeableType, "depositMoney", PlaceableBankAccount.depositMoney)
    SpecializationUtil.registerFunction(placeableType, "withdrawMoney", PlaceableBankAccount.withdrawMoney)
    SpecializationUtil.registerFunction(placeableType, "getBalance", PlaceableBankAccount.getBalance)
    SpecializationUtil.registerFunction(placeableType, "triggerCallback", PlaceableBankAccount.triggerCallback)
    SpecializationUtil.registerFunction(placeableType, "accrueInterest", PlaceableBankAccount.accrueInterest)
end

function PlaceableBankAccount.registerEventListeners(placeableType)
    SpecializationUtil.registerEventListener(placeableType, "onLoad", PlaceableBankAccount)
    SpecializationUtil.registerEventListener(placeableType, "onDelete", PlaceableBankAccount)
    SpecializationUtil.registerEventListener(placeableType, "saveToXMLFile", PlaceableBankAccount)
    SpecializationUtil.registerEventListener(placeableType, "onReadStream", PlaceableBankAccount)
    SpecializationUtil.registerEventListener(placeableType, "onWriteStream", PlaceableBankAccount)
    SpecializationUtil.registerEventListener(placeableType, "onReadUpdateStream", PlaceableBankAccount)
    SpecializationUtil.registerEventListener(placeableType, "onWriteUpdateStream", PlaceableBankAccount)
end

function PlaceableBankAccount:onLoad(savegame)
    print("[PlaceableBankAccount] onLoad вызван")
    self.spec_bankAccount = {}
    local spec = self.spec_bankAccount
    
    local baseXmlPath = "placeable.bankAccount"
    
    -- Загрузка суммы депозита
    spec.depositAmount = self.xmlFile:getValue(baseXmlPath .. "#depositAmount", 5000)
    print("[PlaceableBankAccount] Сумма депозита: " .. spec.depositAmount)
    
    -- Загрузка баланса и последнего дня начисления процентов
    spec.balance = 0
    spec.lastInterestDay = 0
    if savegame ~= nil and savegame.xmlFile ~= nil then
        local key = savegame.key .. "." .. PlaceableBankAccount.modName .. ".bankAccount"
        spec.balance = savegame.xmlFile:getValue(key .. "#balance", 0)
        spec.lastInterestDay = savegame.xmlFile:getValue(key .. "#lastInterestDay", 0)
        print("[PlaceableBankAccount] Загружен баланс из сохранения: " .. spec.balance .. ", Последний день начисления процентов: " .. spec.lastInterestDay)
    end
    
    -- Инициализация флага для синхронизации
    spec.dirtyFlag = self:getNextDirtyFlag()
    print("[PlaceableBankAccount] Инициализирован dirtyFlag: " .. spec.dirtyFlag)
    
    -- Регистрация счёта в BankManager
    if self.isServer then
        BankManager.registerAccount(self)
    end
    
    -- Загрузка триггера
    local triggerNode = self.xmlFile:getValue(baseXmlPath .. "#triggerNode")
    print("[PlaceableBankAccount] triggerNode из XML: " .. tostring(triggerNode))
    if triggerNode then
        local triggerId = I3DUtil.indexToObject(self.components, triggerNode, self.i3dMappings)
        if triggerId then
            print("[PlaceableBankAccount] Регистрация триггера: triggerId=" .. tostring(triggerId))
            addTrigger(triggerId, "triggerCallback", self)
            spec.trigger = triggerId
        else
            print("[PlaceableBankAccount] Ошибка: Узел триггера " .. tostring(triggerNode) .. " не найден в i3d")
        end
    else
        print("[PlaceableBankAccount] Ошибка: triggerNode не определён в XML")
    end
    
    -- Регистрация HUD
    if g_gui then
        if g_gui.guis["BankAccountHUD"] == nil then
            print("[PlaceableBankAccount] Регистрация BankAccountHUD в onLoad")
            if BankAccountHUD and type(BankAccountHUD.register) == "function" then
                local success = BankAccountHUD.register()
                if not success then
                    print("[PlaceableBankAccount] Ошибка: Не удалось зарегистрировать BankAccountHUD")
                    spec.guiLoadFailed = true
                else
                    print("[PlaceableBankAccount] BankAccountHUD зарегистрирован успешно")
                end
            else
                print("[PlaceableBankAccount] Ошибка: BankAccountHUD или его метод register не определён")
                spec.guiLoadFailed = true
            end
        else
            print("[PlaceableBankAccount] BankAccountHUD уже зарегистрирован")
        end
    else
        print("[PlaceableBankAccount] Предупреждение: g_gui не инициализирован")
        spec.guiLoadFailed = true
    end
    
    spec.initialized = true
end

function PlaceableBankAccount:onDelete()
    local spec = self.spec_bankAccount
    if spec.trigger then
        print("[PlaceableBankAccount] Удаление триггера: triggerId=" .. tostring(spec.trigger))
        removeTrigger(spec.trigger)
    end
    if self.isServer then
        BankManager.unregisterAccount(self)
    end
    if g_gui and g_gui.guis["BankAccountHUD"] and BankAccountHUD.INSTANCE and BankAccountHUD.INSTANCE.bank == self then
        BankAccountHUD.INSTANCE:close()
    end
end

function PlaceableBankAccount:saveToXMLFile(xmlFile, key)
    local spec = self.spec_bankAccount
    xmlFile:setValue(key .. "#balance", spec.balance)
    xmlFile:setValue(key .. "#lastInterestDay", spec.lastInterestDay)
    print("[PlaceableBankAccount] Сохранён баланс: " .. spec.balance .. ", Последний день начисления процентов: " .. spec.lastInterestDay)
end

function PlaceableBankAccount:getBalance()
    return self.spec_bankAccount.balance
end

function PlaceableBankAccount:accrueInterest()
    if not self.isServer then return end
    
    local spec = self.spec_bankAccount
    local currentDay = g_currentMission.environment.currentDay
    
    -- Проверяем, сколько дней прошло с последнего начисления
    if spec.lastInterestDay < currentDay then
        local daysPassed = currentDay - spec.lastInterestDay
        local totalInterest = 0
        
        -- Начисляем проценты за каждый пропущенный день
        for day = 1, daysPassed do
            if spec.balance > 0 then
                local dailyInterest = math.floor(spec.balance * 0.05) -- 5% за день, округляем вниз
                spec.balance = spec.balance + dailyInterest
                totalInterest = totalInterest + dailyInterest
                print("[PlaceableBankAccount] Начислены проценты за день " .. (spec.lastInterestDay + day) .. ": " .. dailyInterest .. ", Новый баланс: " .. spec.balance)
            end
        end
        
        if totalInterest > 0 then
            g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, string.format(g_i18n:getText("ui_bankAccount_interestAdded"), totalInterest))
            
            -- Отмечаем изменения для синхронизации
            self:raiseDirtyFlags(spec.dirtyFlag)
        end
        
        spec.lastInterestDay = currentDay
        print("[PlaceableBankAccount] День обновлён, lastInterestDay: " .. spec.lastInterestDay)
    end
end

function PlaceableBankAccount:depositMoney()
    local spec = self.spec_bankAccount
    local farmId = self:getOwnerFarmId()
    local farm = g_farmManager:getFarmById(farmId)
    
    if farm and farmId and farmId > 0 then
        local money = farm.money
        if money < spec.depositAmount then
            print("[PlaceableBankAccount] Недостаточно денег. Требуется: " .. spec.depositAmount .. ", Доступно: " .. money)
            g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, string.format(g_i18n:getText("ui_bankAccount_notEnoughMoney"), spec.depositAmount))
            return false
        end
    else
        print("[PlaceableBankAccount] Ошибка: Неверный farmId (farmId: " .. tostring(farmId) .. ")")
        return false
    end
    
    g_currentMission:addMoney(-spec.depositAmount, farmId, MoneyType.OTHER, true, true)
    spec.balance = spec.balance + spec.depositAmount
    print("[PlaceableBankAccount] Положено на счёт: " .. spec.depositAmount .. ", Новый баланс: " .. spec.balance)
    g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, string.format(g_i18n:getText("ui_bankAccount_deposited"), spec.depositAmount))
    
    -- Отмечаем изменения для синхронизации
    if self.isServer then
        self:raiseDirtyFlags(spec.dirtyFlag)
    end
    
    return true
end

function PlaceableBankAccount:withdrawMoney()
    local spec = self.spec_bankAccount
    if spec.balance <= 0 then
        print("[PlaceableBankAccount] Счёт пуст, нельзя снять деньги")
        g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, g_i18n:getText("ui_bankAccount_empty"))
        return false
    end
    
    local farmId = self:getOwnerFarmId()
    local farm = g_farmManager:getFarmById(farmId)
    if not farm or not farmId or farmId <= 0 then
        print("[PlaceableBankAccount] Ошибка: Неверный farmId (farmId: " .. tostring(farmId) .. ")")
        return false
    end
    
    local amount = spec.balance
    g_currentMission:addMoney(amount, farmId, MoneyType.OTHER, true, true)
    spec.balance = 0
    print("[PlaceableBankAccount] Снято со счёта: " .. amount .. ", Новый баланс: " .. spec.balance)
    g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, string.format(g_i18n:getText("ui_bankAccount_withdrawn"), amount))
    
    -- Отмечаем изменения для синхронизации
    if self.isServer then
        self:raiseDirtyFlags(spec.dirtyFlag)
    end
    
    return true
end

function PlaceableBankAccount:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    print("[PlaceableBankAccount] Триггер вызван: triggerId=" .. tostring(triggerId) .. ", onEnter=" .. tostring(onEnter))
    local player = g_currentMission.player
    if not player and g_currentMission.playerSystem then
        player = g_currentMission.playerSystem:getPlayerByUserId(g_currentMission:getServerUserId())
        if not player and otherShapeId then
            local obj = g_currentMission.nodeToObject[otherShapeId]
            if obj and obj.isa and obj:isa(Player) then
                player = obj
            end
        end
    end
    if onEnter and player then
        if player.farmId ~= self:getOwnerFarmId() then
            print("[PlaceableBankAccount] Доступ запрещён: фермы не совпадают")
            g_gui:showInfoDialog({text=g_i18n:getText("warning_youDontHaveAccessToThis"), dialogType=DialogElement.TYPE_WARNING})
            return
        end
        
        -- Начисляем проценты за пропущенные дни при входе в триггер (на случай, если BankManager не сработал)
        self:accrueInterest()
        
        if g_gui.guis["BankAccountHUD"] then
            if BankAccountHUD.INSTANCE then
                print("[PlaceableBankAccount] Открытие BankAccountHUD для банка: " .. tostring(self))
                BankAccountHUD.INSTANCE.bank = self
                g_gui:showDialog("BankAccountHUD")
            else
                print("[PlaceableBankAccount] Ошибка: Экземпляр BankAccountHUD не найден")
                g_gui:showInfoDialog({text=g_i18n:getText("ui_bankAccount_hudError"), dialogType=DialogElement.TYPE_ERROR})
            end
        else
            print("[PlaceableBankAccount] Ошибка: BankAccountHUD не зарегистрирован")
            g_gui:showInfoDialog({text=g_i18n:getText("ui_bankAccount_hudError"), dialogType=DialogElement.TYPE_ERROR})
        end
    end
end

function PlaceableBankAccount:onReadStream(streamId, connection)
    local spec = self.spec_bankAccount
    spec.balance = streamReadFloat32(streamId)
    spec.lastInterestDay = streamReadInt32(streamId)
    print("[PlaceableBankAccount] Прочитано состояние с сервера: balance=" .. spec.balance .. ", lastInterestDay=" .. spec.lastInterestDay)
end

function PlaceableBankAccount:onWriteStream(streamId, connection)
    local spec = self.spec_bankAccount
    streamWriteFloat32(streamId, spec.balance)
    streamWriteInt32(streamId, spec.lastInterestDay)
    print("[PlaceableBankAccount] Отправлено состояние клиентам: balance=" .. spec.balance .. ", lastInterestDay=" .. spec.lastInterestDay)
end

function PlaceableBankAccount:onReadUpdateStream(streamId, timestamp, connection)
    if not connection:getIsServer() then
        local spec = self.spec_bankAccount
        if streamReadBool(streamId) then
            spec.balance = streamReadFloat32(streamId)
            spec.lastInterestDay = streamReadInt32(streamId)
            print("[PlaceableBankAccount] Обновлено состояние с сервера: balance=" .. spec.balance .. ", lastInterestDay=" .. spec.lastInterestDay)
        end
    end
end

function PlaceableBankAccount:onWriteUpdateStream(streamId, connection, dirtyMask)
    if connection:getIsServer() then
        local spec = self.spec_bankAccount
        if streamWriteBool(streamId, bitAND(dirtyMask, spec.dirtyFlag) ~= 0) then
            streamWriteFloat32(streamId, spec.balance)
            streamWriteInt32(streamId, spec.lastInterestDay)
            print("[PlaceableBankAccount] Отправлено обновление клиентам: balance=" .. spec.balance .. ", lastInterestDay=" .. spec.lastInterestDay)
        end
    end
end